/*******************************************************************
  Components for embedded applications builded for
  laboratory and medical instruments firmware  
 
  flashprog.h - universal FLASH programming module interface 
 
  Copyright (C) 2001-2003 by Pavel Pisa pisa@cmp.felk.cvut.cz
            (C) 2001-2003 by PiKRON Ltd. http://www.pikron.com

 *******************************************************************/

typedef struct flash_alg_info {
  __u32 addr_mask;	/* Mask to take offset inside flash */
  __u32 reg1_addr;	/* Flash control register 1 */
  __u32 reg2_addr;	/* Flash control register 2 */
  __u32	sec_size;	/* block size of bigger blocks */
  __u16	width8;		/* 0 .. 16 bit data, 1 .. 8 bit data bus, 2 ..
			   inerleaved 8 bit */
  __u16 cmd_unlock1;	/* first byte of command sequence */
  __u16 cmd_unlock2;	/* second byte of command sequence */
  __u16 cmd_rdid;	/* read identifier */
  __u16 cmd_prog;	/* program one loc */
  __u16 cmd_erase;	/* erase command */
  __u16 cmd_reset;	/* leave program mode */
  __u16 erase_all;	/* erase all */
  __u16 erase_sec;	/* erase sector */
  __u16 fault_bit;	/* programing of location failed */
  __u16	manid;		/* manufacturer ID */
  __u16 devid;		/* device ID */
} flash_alg_info_t;

int flash_check_id(flash_alg_info_t const *alg, void *addr,
		   __u16 retid[2]);

int flash_prog(flash_alg_info_t const *alg, void *addr, __u16 val);

int flash_erase(flash_alg_info_t const *alg, void *addr, long size);

flash_alg_info_t const amd_29f400_x16;

flash_alg_info_t const amd_29f800_x16;
