#include <types.h>
#include <system_def.h>
#include <sim.h>
#include <ctm4.h>
#include <qadc.h>
#include <string.h>

/********************************************************************/
/* PWM and SF interrupt from CTM4 */


__u8 ctm4_pwm_pdir0s;	/* pwm direction shadow registers */
__u8 ctm4_pwm_pdir1s;

unsigned ctm4_sfi_cnt=0;
unsigned ctm4_sfi_prolong=0;
unsigned ctm4_sfi_promax=0;
unsigned ctm4_sfi_spent=0;
unsigned ctm4_sfi_spmax=0;

/* sampling frequency interrupt handler */
void ctm4_sfi_isr(int intno, void *dev_id, struct pt_regs *regs)
{
  *CTM4_MCSM2SIC&=~CTM4_MCSM_COF;

  ctm4_sfi_prolong=*CTM4_MCSM2CNT;
  if(ctm4_sfi_prolong>ctm4_sfi_promax)
    ctm4_sfi_promax=ctm4_sfi_prolong;
  
  ctm4_sfi_cnt++;
  msec_time++;

  ctm4_sfi_spent=*CTM4_MCSM2CNT-ctm4_sfi_prolong;
  if(ctm4_sfi_spent>ctm4_sfi_spmax)
    ctm4_sfi_spmax=ctm4_sfi_spent;
  
}


irq_handler_t ctm4_sfi_handler={
  handler: ctm4_sfi_isr,
  flags:   0,
  dev_id:  0,
  devname: "ctm4_sfi",
  next:    0
};


int ctm4_time_init()
{
  /* BIUMCR - BIU Module Configuration Register */
  *CTM4_BIUMCR=(0*CTM4_STOP)|(CTM4_VECT&(ISR_CTM4_INTV<<5))|
               (CTM4_IARB&(ISR_CTM4_IARB<<8));
  
  /* CPCR - CPSM Control Register */
  *CTM4_CPCR=CTM4_PRUN|(0*CTM4_DIV23)|(CTM4_PSEL&0);
      
  /* PWM5 initialize */
  *CTM4_PWM5SIC=CTM4_PWM_POL|CTM4_PWM_EN|(CTM4_PWM_CLK&0);
  *CTM4_PWM5A=512;	/* period */
  *CTM4_PWM5B=0;	/* pulse width */

  /* PWM6 initialize */
  *CTM4_PWM6SIC=CTM4_PWM_POL|CTM4_PWM_EN|(CTM4_PWM_CLK&0);
  *CTM4_PWM6A=512;	/* period */
  *CTM4_PWM6B=0;	/* pulse width */

  /* PWM7 initialize */
  *CTM4_PWM7SIC=CTM4_PWM_POL|CTM4_PWM_EN|(CTM4_PWM_CLK&0);
  *CTM4_PWM7A=512;	/* period */
  *CTM4_PWM7B=0;	/* pulse width */

  /* PWM8 initialize */
  *CTM4_PWM8SIC=CTM4_PWM_POL|CTM4_PWM_EN|(CTM4_PWM_CLK&0);
  *CTM4_PWM8A=512;	/* period */
  *CTM4_PWM8B=0;	/* pulse width */

  /* FCSM12 initialize */
  *CTM4_FCSM12SIC=CTM4_FSCM_DRVB|(CTM4_FSCM_CLK&0);

  /* DASM3 initialize */
  *CTM4_DASM3SIC=(CTM4_DASM_BSL*1)|CTM4_DASM_EDPOL|CTM4_DASM_PWM9;
  *CTM4_DASM3A=0;	/* pulse start */
  *CTM4_DASM3B=0;	/* pulse end */

  /* DASM4 initialize */
  *CTM4_DASM4SIC=(CTM4_DASM_BSL*1)|CTM4_DASM_EDPOL|CTM4_DASM_PWM9;
  *CTM4_DASM4A=0;	/* pulse start */
  *CTM4_DASM4B=0;	/* pulse end */

  /* DASM9 initialize */
  *CTM4_DASM9SIC=(CTM4_DASM_BSL*1)|CTM4_DASM_EDPOL|CTM4_DASM_PWM9;
  *CTM4_DASM9A=0;	/* pulse start */
  *CTM4_DASM9B=0;	/* pulse end */

  /* DASM10 initialize */
  *CTM4_DASM10SIC=(CTM4_DASM_BSL*1)|CTM4_DASM_EDPOL|CTM4_DASM_PWM9;
  *CTM4_DASM10A=0;	/* pulse start */
  *CTM4_DASM10B=0;	/* pulse end */
  
  /* Enable power direction register output */
  *QADC_DDRQA|=0x8000;
  *QADC_PORTQ&=~0x8000;

  /* initialize sampling frequency interrupt */
  *CTM4_MCSM2ML=-((cpu_sys_hz+CTM4_SFI_HZ)/(CTM4_SFI_HZ*2));
  *CTM4_MCSM2SIC=(CTM4_MCSM_IL&(ISR_CTM4_SFI_IL<<12))|
                 (ISR_CTM4_IARB&8?CTM4_MCSM_IARB3:0)|CTM4_MCSM_DRVA|
                 (CTM4_MCSM_CLK&0);
  add_irq_handler(ISR_CTM4_SFI_INTV,&ctm4_sfi_handler);

  return 0;
}

